local nItemsPerTab = 98
local nScanTimeStamp
local bTabScanned = {}
local bDoneScan = false
local bDoScan = false

local nVersion = 1

GUILD_BANK_TIMER_ARRAY = {}
GUILD_BANK_TIMER_VERSION = nVersion

local CurrentBankData = nil
local CurrentBankQuantities = nil

local function InitializeScan()
	
	local nBankTabs = GetNumGuildBankTabs()
	
	bDoneScan = false
	bDoScan = true
	
	for nLoop = 1, nBankTabs do		
		QueryGuildBankTab( nLoop )		
		bTabScanned[nLoop] = false
	end
	
end

local function GetCurrentTimeStamp()
	local nCurrHour, nCurrMinute = GetGameTime()
	local _, nCurrMonth, nCurrDay, nCurrYear = CalendarGetDate()
	
	local nMonths = nCurrYear * 12 + nCurrMonth
	-- To calculate timestamp, months are all considered to have 32 days.
	local nDays = nMonths * 32 + nCurrDay
	local nHours = nDays * 24 + nCurrHour
	local nMinutes = nHours * 60 + nCurrMinute		
	
	return nMinutes
end

local function IterateOnTabs()
	if (bDoScan ~= true) then
		return
	end
	if (bDoneScan == true) then
		return
	end
	
	local sLink
	local nTabs = GetNumGuildBankTabs()

	for nTab = 1, nTabs do
		if (bTabScanned[nTab] ~= true) then
			for nSlot= 1, nItemsPerTab do 
				sLink = GetGuildBankItemLink(nTab, nSlot)
				if sLink then 
					--tab has an item
					bTabScanned[nTab] = true
					break
				end
			end			
		end
	end

	for nTab = 1, nTabs do
		if (bTabScanned[nTab] ~= true) then
			-- we're not done.  stop here
			return
		end
	end
	
	local sGuild = GetGuildInfo("player")
	
	if (sGuild == nil) then
		return
	end
	
	CurrentBankData = GUILD_BANK_TIMER_ARRAY[sGuild]
	if (CurrentBankData == nil) then
		-- create a new entry
		CurrentBankData = {}
		GUILD_BANK_TIMER_ARRAY[sGuild] = CurrentBankData
	else
		
		local nTotal = 0
		local LastValue
		for key, value in pairs(CurrentBankData) do
			nTotal = nTotal + 1
			LastValue = value
		end
	end
		
	nScanTimeStamp = GetCurrentTimeStamp()
	
	local NewArray = {}
	CurrentBankQuantities = {}
	
	-- All tabs are scanned.  Complete the scan
	for nTab = 1, nTabs do
		for nSlot= 1, nItemsPerTab do 
			sLink = GetGuildBankItemLink(nTab, nSlot)
			if (sLink ~= nil) then 
				local itemName = GetItemInfo( sLink )
				if (itemName ~= nil) then
					NewArray[itemName] = nScanTimeStamp
					local nCurrQuantity = CurrentBankQuantities[itemName] or 0
					nCurrQuantity = nCurrQuantity + 1
					CurrentBankQuantities[itemName] = nCurrQuantity
				end
				
			end
		end
	end
	
	-- Now scan everything in (old) CurrentBankData.  	
	local nPreviousCount = 0
	for key, value in pairs(CurrentBankData) do
		nPreviousCount = nPreviousCount + 1
		if (NewArray[key] ~= nil) then
			-- item existed before, retain old time stamp
			NewArray[key] = value
		end
	end
		
	CurrentBankData = NewArray
	GUILD_BANK_TIMER_ARRAY[sGuild] = NewArray
	
	local nCount = 0
	local nOldCount = 0
	for key, value in pairs(CurrentBankData) do
		nCount = nCount + 1
		if (value <= nScanTimeStamp) then
			nOldCount = nOldCount + 1
		end
	end
	
	bDoneScan = true
	bDoScan = false
end

-- 
-- Hook into tooltip
-- 

local sAgePrompt = "Item has been seen in bank for"
local sQuantityPrompt = "Number (stacks) in bank"

local function AddInfoToTooltip(tip, sLink)
	if (bDoneScan == true) then
		-- Get the name
		local sName = GetItemInfo( sLink )
		
		-- Now look it up
		local nTimeSeen = CurrentBankData[sName]
		if (nTimeSeen ~= nil) then	
			local nCurrTimeStamp = GetCurrentTimeStamp()
			local nDiff = nCurrTimeStamp - nTimeSeen
					
			local sValue
			
			if (nDiff < 0) then
				sValue = "error!"
			elseif (nDiff < 60) then
				sValue = tostring(nDiff) .. "m"
			elseif (nDiff < 1440) then
				local nMinutes = nDiff % 60
				local nHours = (nDiff - nMinutes) / 60
				
				sValue = tostring(nHours) .. "h" .. tostring(nMinutes) .. "m"
			else
				local nMinutes = nDiff % 1440
				local nDays = (nDiff-nMinutes) / 1440
				local nHours = math.floor( nMinutes / 60 )
				sValue = tostring(nDays) .. "d" .. tostring(nHours) .. "h"
			end
			tip:AddDoubleLine(sAgePrompt, sValue, 0, .8, 1, 1, 1, 1)
			
		end
		
		local nQuantity = CurrentBankQuantities[sName]
		if (nQuantity ~= nil) then
			local sValue = tostring(nQuantity)
			tip:AddDoubleLine(sQuantityPrompt, sValue, 0, .8, 1, 1, 1, 1)
		end
		
		tip:Show()	
	end
end

local hooks = {
	SetGuildBankItem = function(nTab, nSlot)
		return GetGuildBankItemLink(nTab, nSlot)
	end,	
	
}

local function hookTip(tooltip, method, action)
	hooksecurefunc(tooltip, method, function(tip, ...)
		local link = action(...)
		if link then
			AddInfoToTooltip(tip, link)
		end
	end)
end


function GuildBankTimer_OnLoad(self)
	self:RegisterEvent("GUILDBANKFRAME_OPENED")
	self:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
	
	for method,func in pairs(hooks) do
		hookTip(GameTooltip, method, func)
	end	
end

function GuildBankTimer_OnEvent(self, event)
	if (event == "GUILDBANKFRAME_OPENED") then
		InitializeScan()
	elseif (event == "GUILDBANKBAGSLOTS_CHANGED") then
		IterateOnTabs()
	end
	
end
